FUNCTION bootmsini,mstar,f,approx=approx,criterion=criterion

;+
; mplanet,f,approx=approx
;
; Calculate planet mass, max(m*sin(i)), from the mass function f and
; stellar mass Mstar.
;
; INPUT VALUE:
;    f - mass function value(s) in grams.
;    mstar - host star stellar mass in solar mass.
;    criterion - converging criterion, (percentage of difference
;                between thisMp and nextMp)*100. Default = 1.0d-6
;                (meaning 0.0001%).
;
; INPUT KEYWORD:
;    approx - if /approx, then return the approximation where Mplanet
;             << Mstar is assumed.
;
; OUTPUT: 
;    returned value - mass of the planet in Jupiter Mass.
;
; HISTORY:
;   Written by Sharon X. Wang, Dec 13 2011
;  
;-   

  ; Constants
  Msun = double(1.98892e33)     ; solar mass in g
  Mjup = double(1.8986e30)      ; Jupiter mass in g
  ; Everything in double
  f = double(f)
  mstar = double(mstar)

  ; Calculate using approximation
  if keyword_set(approx) then begin
     logMsini = double( (alog(f)+2.0*alog(mstar)+2.0*alog(Msun))/3.0-alog(Mjup)) ; with msin in Jupiter M, mstar in solar mass
     msini = exp(logMsini)                                                       ; msini in Jupiter M
     return,msini
  endif

  ; Rigorous calculation begins here:
  allmp = -1
  for i=0,n_elements(f)-1 do begin
     thismp = double( (alog(f[i])+2.0*alog(mstar)+2.0*alog(Msun))/3.0-alog(Mjup))
     thismp = exp(thismp)       ; in Jupiter mass
     nextmp = 0d
     incre = double( abs(((thismp-nextmp)/thismp)) )
     if ~keyword_set(criterion) then crit = 1.0d-6 else crit = criterion
     while (incre ge crit) do begin
        lognext = double( (2.0*alog(mstar*Msun+thismp*Mjup)+alog(f[i]))/3.0 - alog(Mjup) ) ; with msini in Jupiter Mass
        nextmp = exp(lognext)                                                           ; in Jupiter mass
        incre = double( abs(((thismp-nextmp)/thismp)) )
        thismp = nextmp
     endwhile
     allmp = [allmp,thismp]
  endfor

  allmp = allmp[where(allmp ne -1)]
  
  return,allmp
  
END 
