PRO bootpar,tve1,tve2,tve3,tve4,tve5,tve6,tele=tele,bestpar=bestpar $ 
           ,bootpar=bootpar,bootnewpar=bootnewpar,bootse=bootse,bootoffset=bootoffset, $
            Tc_planet=Tc_planet,jitter=jitter,_extra=extra, nboot=nboot

;+
;Procedure for bootstrapping to get error bars on orbital
;parameters. Can deal with multiple telescope, either re-fitting
;telescope offsets when bootstrapping, or fixed offsets.
;
;INPUT:
;tve1, tve2 .... tve6 (structure) Input structure. Must have tags JD,
;                                 MNVEL and ERRVEL.
;                     (array)     tve1 = JD array, tve2 = MNVEL array,
;                                 tve3 = ERRVEL array.
;tele (optional array) Telescope labels.
;bestpar (array) Input best fitted parameters, either 7 or 14 elements
;                for 1 or 2 planet cases, respectively.
;Tc_planet (int) Which planet to calculate Tc (newpars) for?
;jitter (double) Stellar jitter in m/s.
;_extra (inherited keywords) keywords or parameters to be inheritated
;                            by rv_fit_mp.
;nboot (optional long/int) number of bootstrap steps / sampling size.
;  
;OUTPUT:
;bootoffset (bootoffset) Output bootstrap results for telescope offsets.
;bootpar (array) Output bootstrap results for planet fit parameters,
;                under same convension as bestpar (see Jason
;                Wright's rvlin package or rv_fit_mp.pro).
;bootnewpar (array) Output bootstrap results for period, transit time
;                   mid-point, ecosw and esinw.
;bootse (array) Output secondary eclipse times
;
;Created by Sharon X. Wang, Dec 2010
;  
; ----------------------------------------------------------------------------
;          Version 1.0   Modification History
; ----------------------------------------------------------------------------    
;  
;Modified to accept general form (multi-tag) of tve1...tve6, by SXW
;Feb 2011
;  
;Modified to accept arrays for jd,vel,err,tele=tele for multiple
;telescope situation. SXW Mar 2011
;  
;Added stellar jitter. SXW May 2011
;  
;Fixed bug regarding scrambling when having telescope indices not in
;chunks but scrambled around. SXW Nov 2011
;  
;Added keyword inheritance for rv_fit_mp, _extra. SXW Jan 2011
;
;Added warning message for Tp not within observation time window.
;SXW Mar 2012
;
;Added nboot parameter to enable increase in bootstrap step.
;SXW Mar 2012
;
;Merged with EOD version containing secondary eclipse calculation
;SXW JTW April 23 2012
;
; ----------------------------------------------------------------------------
;  
;-  

  ; Fitting with trend (dvdt)?
  if bestpar[6] ne 0.0 then trend = 1 else trend = 0
  if 1-keyword_set(Tc_planet) then Tc_planet = 1
  tcp = Tc_planet-1

  
  ; Check the Tp in bestpar, see if they lie within observation window
  ; Future: move Tp into observation time window?
  if datatype(tve1) eq 'STC' then begin
     obsjds = [tve1.jd,keyword_set(tve2)?tve2.jd:-1,keyword_set(tve3)?tve3.jd:-1, $
               keyword_set(tve4)?tve4.jd:-1,keyword_set(tve5)?tve5.jd:-1, $
               keyword_set(tve6)?tve6.jd:-1]
     obswindow = mm(obsjds[where(obsjds ne -1)])
  endif else begin
     obswindow = mm(tve1) ; min and max of input BJD/JD
  endelse
  for i=0,(n_elements(bestpar)/7-1) do begin
     if bestpar[i*7+1] lt obswindow[0] or bestpar[i*7+1] gt obswindow[1] then begin
        print,'%% BOOTPAR: !!!!!!!!!!!!!!!!!!!!  WARNING  !!!!!!!!!!!!!!!!!!!!'
        print,'%% BOOTPAR: It is recommended that your best-fit Tp (time of periastron) '
        print,'            falls within the observation time window.'
        print,'%% BOOTPAR: Here bestpar['+strtrim(i*7+1,2)+'] is outside of obs window!'
        print,'%% BOOTPAR: !!!!!!!!!!!!!!!!!!!!  WARNING  !!!!!!!!!!!!!!!!!!!!'
        print,''
     endif 
  endfor

 
  ; Set up tve structure for bootpar
  if datatype(tve1) ne 'STC' then begin
     dum = {JD:0.0D,MNVEL:0.0D,ERRVEL:0.0D}
     tve = replicate(dum,n_elements(tve1))
     tve.jd = tve1              ; never modify input tve's
     tve.mnvel = tve2
     tve.errvel = tve3
     if 1-keyword_set(tele) then begin
        ntele = 1
     endif else begin
        tele = strupcase(strtrim(string(tele),2))
        teleind = tele(rem_dup(tele))
        ntele = n_elements(teleind)
        if ntele gt 6 then begin
           print,'%% BOOTPAR: Sorry, cant deal with more than 6 telescopes. :S'
           stop
        endif
     endelse 
  endif else begin
     case 1 of ; Determine number of telescopes
        1-keyword_set(tve2): ntele=1
        1-keyword_set(tve3): ntele=2
        1-keyword_set(tve4): ntele=3
        1-keyword_set(tve5): ntele=4
        1-keyword_set(tve6): ntele=5
        else: ntele=6
     endcase    
     if ntele eq 1 then tve = tve1
  endelse 

  if ntele eq 1 then begin
     print,'%% BOOTPAR: Only 1 telescope OR using fixed best-fit offsets.'

     if trend eq 1 then begin
        if tve[0].jd gt 20000.D then begin
           print,'%% BOOTPAR: WARNING!!!! - Please use CPS JD (JD-2440000) when fitting with a trend!'
           print,'%% BOOTPAR: Please modify your JD and come back again. Thanks.'
           retall
        endif
        if median(tve.mnvel) gt (max(tve.mnvel)-min(tve.mnvel)) then begin
           print,'%% BOOTPAR: WARNING!!!! - To ensure best service quality, please re-scale'+ $
                 ' your velocities by subtracting their median or mean off.'
           print,'%% BOOTPAR: Please modify your velocities and come back again. Thanks.'
           retall
        endif 
     endif

     bestfit = rv_drive(tve.jd,bestpar)
     residuals = tve.mnvel - bestfit
     npt = n_elements(residuals)                ; number of points on velocity plot
     npar = n_elements(bestpar)                 ; number of parameters
     if ~keyword_set(nboot) then nboot = floor(max([1000,npt*alog10(npt)^2])) $
     else nboot = floor(max([1000,npt*alog10(npt)^2,nboot])) ; number of bootstrapping loops
     bootpar = dblarr(nboot,npar)                 ; to store the bootstrapping results
     bootnewpar = dblarr(nboot,4)                 ; this one has transit mid point JD time
     bootse = dblarr(nboot)                       ; this one has SE mid point JD time
     tvetemp = tve
     for i=0,nboot-1 do begin
        if ~(i mod 100) then counter,i+1,nboot
        scramble = randomu(seed,npt)*npt
        tvetemp.mnvel = residuals[scramble] + bestfit ; scramble the residuals
        tvetemp.errvel = tve[scramble].errvel        ; errvel goes with original residual
        bootpar[i,*] = rv_fit_mp(tvetemp,orbel=bestpar,trend=trend,/quiet,jitter=jitter,_extra=extra)
        bootnewpar[i,*]=newpars(bootpar[i,0],bootpar[i,2],bootpar[i,3],bootpar[i,1])
        bootse[i]=separs(bootpar[i,0],bootpar[i,2],bootpar[i,3],bootpar[i,1])
     endfor
     print,''
  endif else begin
     print,'%% BOOTPAR: Multiple telescopes - solving for offsets every time.'
     print,'%% BOOTPAR: Scrambling residuals within each telescope.'
     cfsample = {jd:0.0D,mnvel:0.0D,errvel:0.0D}
     
     if datatype(tve1) ne 'STC' then begin
        tve_all = replicate(cfsample,n_elements(tve1))
        tve_all.jd = tve1
        tve_all.mnvel = tve2
        tve_all.errvel = tve3
     endif else begin
        for i=1,ntele do begin
           CASE 1 OF
              i eq 1: BEGIN
                 tvejunk = replicate(cfsample,n_elements(tve1))
                 tvejunk.jd = tve1.jd
                 tvejunk.mnvel = tve1.mnvel
                 tvejunk.errvel = tve1.errvel
                 tve_all = tvejunk
                 tele = strtrim(intarr(n_elements(tve1))+1,2)
              END
              i eq 2: BEGIN
                 tvejunk = replicate(cfsample,n_elements(tve2))
                 tvejunk.jd = tve2.jd
                 tvejunk.mnvel = tve2.mnvel
                 tvejunk.errvel = tve2.errvel
                 tve_all = [tve_all,tvejunk]
                 tele = [tele,strtrim(intarr(n_elements(tve2))+2,2)]
              END
              i eq 3: BEGIN
                 tvejunk = replicate(cfsample,n_elements(tve3))
                 tvejunk.jd = tve3.jd
                 tvejunk.mnvel = tve3.mnvel
                 tvejunk.errvel = tve3.errvel
                 tve_all = [tve_all,tvejunk]
                 tele = [tele,strtrim(intarr(n_elements(tve3))+3,2)]
              END
              i eq 4: BEGIN
                 tvejunk = replicate(cfsample,n_elements(tve4))
                 tvejunk.jd = tve4.jd
                 tvejunk.mnvel = tve4.mnvel
                 tvejunk.errvel = tve4.errvel
                 tve_all = [tve_all,tvejunk]
                 tele = [tele,strtrim(intarr(n_elements(tve4))+4,2)]
              END
              i eq 5: BEGIN
                 tvejunk = replicate(cfsample,n_elements(tve5))
                 tvejunk.jd = tve5.jd
                 tvejunk.mnvel = tve5.mnvel
                 tvejunk.errvel = tve5.errvel
                 tve_all = [tve_all,tvejunk]
                 tele = [tele,strtrim(intarr(n_elements(tve5))+5,2)]
              END
              i eq 6: BEGIN
                 tvejunk = replicate(cfsample,n_elements(tve6))
                 tvejunk.jd = tve6.jd
                 tvejunk.mnvel = tve6.mnvel
                 tvejunk.errvel = tve6.errvel
                 tve_all = [tve_all,tvejunk]
                 tele = [tele,strtrim(intarr(n_elements(tve6))+6,2)]
              END
              ELSE: BEGIN
                 print,'%% BOOTPAR: This should never happen.'
                 stop
              END 
           ENDCASE
        endfor
     endelse 

     if trend eq 1 then begin
        if tve_all[0].jd gt 20000.D then begin
           print,'%% BOOTPAR: WARNING!!!! - Please use CPS JD (JD-2440000) when fitting with a trend!'
           print,'%% BOOTPAR: Please modify your JD and come back again. Thanks.'
           retall
        endif
        if median(tve_all.mnvel) gt (max(tve_all.mnvel)-min(tve_all.mnvel)) then begin
           print,'%% BOOTPAR: WARNING!!!! - To ensure best service quality, please re-scale'+ $
                 ' your velocities by subtracting their median or mean off.'
           print,'%% BOOTPAR: Please modify your velocities and come back again. Thanks.'
           retall
        endif
     endif
     
     bestfit = rv_drive(tve_all.jd,bestpar)
     residuals = tve_all.mnvel - bestfit
     npt = n_elements(residuals)                ; number of points on velocity plot
     npar = n_elements(bestpar)                 ; number of parameters
     if ~keyword_set(nboot) then nboot = floor(max([1000,npt*alog10(npt)^2])) $
     else nboot = floor(max([1000,npt*alog10(npt)^2,nboot])) ; number of bootstrapping loops
     bootpar = dblarr(nboot,npar)                 ; to store the bootstrapping results
     bootnewpar = dblarr(nboot,4)                 ; this one has transit middle point JD time
     bootse = dblarr(nboot)                       ; this one has eclipse midpoint JD time
     bootoffset = dblarr(nboot,ntele-1)           ; to store bootstrap telescope offsets
     tvetemp = tve_all
                                ; METHOD OF BOOTSTRAP: DRAW WITH REPLACEMENT (not really scramble)
     for i=0,nboot-1 do begin
        if ~(i mod 100) then counter,i+1,nboot
        if n_elements(teleind) le 0 then teleind = strtrim(indgen(ntele)+1,2)  ; 1,2,...,ntele, string array
        varind = indgen(npt)                              ; indices of variables
        scramble = indgen(npt)                            ; scrambled indices
        for j=0,ntele-1 do begin                          ; scramble within each telescope
           scramble_ind = varind(where(tele eq teleind[j])) ; only scramble these ones
           scrambleMe = indgen(n_elements(scramble_ind))    ; scramble the indices of scramble_ind - fixed bug SXW Nov 23 2011
           scrambled = (randomu(seed,n_elements(scrambleMe))*(max(scrambleMe)-min(scrambleMe)+1)+min(scrambleMe) $
                        <max(scrambleMe)) >min(scrambleMe) ; this has to be done this way because scramble_ind could be [1,4,6,10] instead of neatly [3,4,5,6]
           scrambled = floor(scrambled)
           scrambled = scramble_ind[scrambled]
           scramble[scramble_ind] = scrambled
        endfor    
        ;; scramble = randomu(seed,npt)*npt ;; scramble between telescopes
                                            ;; bad way to bootstrap
        tvetemp.mnvel = residuals[scramble] + bestfit ; scramble the residuals
        tvetemp.errvel = tve_all[scramble].errvel     ; errvel goes with original residual
        bootpar[i,*] = rv_fit_mp(tvetemp.jd,tvetemp.mnvel,tvetemp.errvel $
                                 ,telvec=tele,offset=offset,orbel=bestpar,trend=trend,/quiet,jitter=jitter,_extra=extra)
        ; Calculate newpars for Planet # Tc_planet (tcp+1)
        bootnewpar[i,*]=newpars(bootpar[i,0+tcp*7],bootpar[i,2+tcp*7],bootpar[i,3+tcp*7],bootpar[i,1+tcp*7])
        bootse[i]=separs(bootpar[i,0+tcp*7],bootpar[i,2+tcp*7],bootpar[i,3+tcp*7],bootpar[i,1+tcp*7])
        bootoffset[i,*]=offset
     endfor
     print,''
  endelse 



END
